#-----------------------------------------------------------------------------
# Oscillator
# Copyright (c) 2006 by Ockham (David G. Drumright)
# May 7, 2006
# Revised Oct 12, 2009 to use arrays instead of files.
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser

global wavemode
WAVE_SINE=0
WAVE_SQUARE=1
WAVE_TRIANGLE=2
wavemode=WAVE_SINE


scene = poser.Scene()
TheFig=scene.CurrentFigure()
if not TheFig: raise 'No figure!'

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Oscillator")
		self.Diffs1=[]
		self.Diffs2=[]
		self.ChosenList1=[]
		self.ChosenList2=[]

		self.StatusEntry = Entry(self.master,width=30)
		self.StatusEntry.grid(row=0,column=0,columnspan=3,padx=2,pady=2)
		self.StatusEntry.insert(0,"Set one end of vibration, hit 'Mark 1'")

		# Go/nogo buttons.
		self.buttonMark1 = Button(self.master, text="Mark 1", command=self.HandleMark1)
		self.buttonMark1.grid(row=2, column=0,padx=2,pady=2)

		self.buttonMark2 = Button(self.master, text="Mark 2", command=self.HandleMark2)
		self.buttonMark2.grid(row=2, column=2,padx=2,pady=2)

		Label(self.master,text="Start frame:").grid(row=3,column=0,padx=2,pady=2)
		self.StartEntry = Entry(self.master,width=6)
		self.StartEntry.grid(row=3,column=2,padx=2,pady=2)

		Label(self.master,text="End frame:").grid(row=4,column=0,padx=2,pady=2)
		self.EndEntry = Entry(self.master,width=6)
		self.EndEntry.grid(row=4,column=2,padx=2,pady=2)

		Label(self.master,text="Cycles per second:").grid(row=5,column=0,padx=2,pady=2)
		self.FreqEntry = Entry(self.master,width=6)
		self.FreqEntry.grid(row=5,column=1,padx=2,pady=2)

		Label(self.master,text="Damping (0 for auto):").grid(row=6,column=0,padx=2,pady=2)
		self.DampEntry = Entry(self.master,width=6)
		self.DampEntry.grid(row=6,column=1,padx=2,pady=2)



		self.buttonSine = Button(self.master, text="Sine", command=self.HandleSine)
		self.buttonSine.grid(row=7, column=0,padx=2,pady=2)
		self.buttonSquare = Button(self.master, text="Square", command=self.HandleSquare)
		self.buttonSquare.grid(row=7, column=1,padx=2,pady=2)
		self.buttonTriangle = Button(self.master, text="Triangle", command=self.HandleTriangle)
		self.buttonTriangle.grid(row=7, column=2,padx=2,pady=2)


		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


	# - - - - - - - - - - - - - - - - - -

	def HandleMark1(self):
		self.ChosenList1=[]
		# Each member of list is tuple: figname, partname, parmname, value
		# If no fig, None.
		for A in TheFig.Actors():
			for P in A.Parameters():
				if P.Hidden(): continue
				self.ChosenList1.append( (TheFig.Name(),A.Name(),P.Name(),P.Value() ) )
		self.ShowStatus("Now set other end of vibration, hit 'Mark 2'")

	# - - - - - - - - - - - - - - - - - -

	def HandleMark2(self):
		self.ChosenList2=[]
		# Each member of list is tuple: figname, partname, parmname, value
		# If no fig, None.
		for A in TheFig.Actors():
			for P in A.Parameters():
				if P.Hidden(): continue
				self.ChosenList2.append( (TheFig.Name(),A.Name(),P.Name(),P.Value() ) )
		self.ShowStatus("Now set other values, then pick a Wave.")

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -

	def Update(self):
		if scene: scene.ProcessSomeEvents(1)
		root.lift()
		root.after(100, self.Update)

	# - - - - - - - - - - - - - - - - - -
	def HandleSine(self):
		global wavemode
		wavemode=WAVE_SINE
		self.HandleGo()
	# - - - - - - - - - - - - - - - - - -
	def HandleSquare(self):
		global wavemode
		wavemode=WAVE_SQUARE
		self.HandleGo()
	# - - - - - - - - - - - - - - - - - -
	def HandleTriangle(self):
		global wavemode
		wavemode=WAVE_TRIANGLE
		self.HandleGo()
	# - - - - - - - - - - - - - - - - - -

	def HandleGo(self):
		# Action here
		# Pick up frequency
		e=self.FreqEntry.get()
		if e: Freq = float(e)
		else: Freq = 1.0  # Arb one per second
		if Freq == 0.0: Freq = 1.0

		e=self.StartEntry.get()
		if e: Start = int(e)
		else: Start=1

		e=self.EndEntry.get()
		if e: End = int(e)
		else: End = scene.NumFrames()
		if End <= Start:
			self.ShowStatus("End must be after Start!")
			return

		FramesPerHalf = int(0.5*float(scene.FramesPerSecond())/Freq)
		if FramesPerHalf == 0: FramesPerHalf=1

		e=self.DampEntry.get()
		if e: Damp = float(e)
		else: Damp = 1.0  # Default to no decay
		if Damp == 0.0: # Automatic: Find a damping constant that
			# will reduce the motion to 0.01 (because 0 gets math trouble)
			# over the interval.  Note that we're using the number of STEPS
			# here, not the number of FRAMES.
			Frames = float(End-Start+1)
			Steps = Frames / float(FramesPerHalf)
			Damp = pow(0.01, (1.0 / Steps))

		Use2=0
		Netdamp=1.0
		self.PickUpDiffs(wavemode,Start,End)

		if wavemode==WAVE_SQUARE: # hold each up to the next division
			for F in range(Start,End,FramesPerHalf):
				if Use2:
					self.Recall2(Netdamp,F,(F+FramesPerHalf))
				else:
					self.Recall1(Netdamp,F,(F+FramesPerHalf))
				Use2 = 1-Use2 # Toggle
				Netdamp = Netdamp * Damp

		else: # sine or tri
			for F in range(Start,End,FramesPerHalf):

				if Use2:
					self.Recall2(Netdamp,F,(F+1))
				else:
					self.Recall1(Netdamp,F,(F+1))
				Use2 = 1-Use2 # Toggle
				Netdamp = Netdamp * Damp



		self.ShowStatus("Done.")

	# - - - - - - - - - - - - - - - - - -

	def PickUpDiffs(self,wavemode,Start,End):
		self.Diffs1=[]
		self.Diffs2=[]

		for (FigName1,PartName1,ParmName1,ParmValue1) in self.ChosenList1:
			for (FigName2,PartName2,ParmName2,ParmValue2) in self.ChosenList2:
				if FigName1 == FigName2:
					if PartName1 == PartName2:
						if ParmName1 == ParmName2:
							if ParmValue1 != ParmValue2:
								self.Diffs1.append((FigName1,PartName1,ParmName1,ParmValue1))
								self.Diffs2.append((FigName2,PartName2,ParmName2,ParmValue2))
								# do the linear/spline here.
								if FigName1:
									try:
										Fig=scene.Figure(FigName1)
									except:
										continue
									try:
										A=Fig.Actor(PartName1)
									except:
										continue
									try:
										P=A.Parameter(ParmName1)
										if wavemode != WAVE_SINE:
											P.SetRangeLinear(Start,End)
										else:
											P.SetRangeSpline(Start,End)
									except:
										continue
								else: # Items outside a fig, we put None in this element
									try:
										A=scene.Actor(PartName1)
									except:
										continue
									try:
										P=A.Parameter(ParmName1)
										if wavemode != WAVE_SINE:
											P.SetRangeLinear(Start,End)
										else:
											P.SetRangeSpline(Start,End)
									except:
										continue




	# - - - - - - - - - - - - - - - - - -

	def Recall1(self,damp,sub1,sub2):
		# Fig names are pref'd by Fig.
		for F in range(sub1,sub2): # will be same for sine/tri, a real range for square
			scene.SetFrame(F-1)
			for (FigName,PartName,ParmName,ParmValue) in self.Diffs1:
				if FigName:
					try:
						Fig=scene.Figure(FigName)
					except:
						continue
					try:
						A=Fig.Actor(PartName)
					except:
						continue
					try:
						P=A.Parameter(ParmName)
						P.SetValue(ParmValue*damp)
					except:
						continue
				else: # Items outside a fig, we put None in this element
					try:
						A=scene.Actor(PartName)
					except:
						continue
					try:
						P=A.Parameter(ParmName)
						P.SetValue(ParmValue*damp)
					except:
						continue

	# - - - - - - - - - - - - - - - - - -

	def Recall2(self,damp,sub1,sub2):
		# Fig names are pref'd by Fig.
		for F in range(sub1,sub2): # will be same for sine/tri, a real range for square
			scene.SetFrame(F-1)
			for (FigName,PartName,ParmName,ParmValue) in self.Diffs2:
				if FigName:
					try:
						Fig=scene.Figure(FigName)
					except:
						continue
					try:
						A=Fig.Actor(PartName)
					except:
						continue
					try:
						P=A.Parameter(ParmName)
						P.SetValue(ParmValue*damp)
					except:
						continue
				else: # Items outside a fig, we put None in this element
					try:
						A=scene.Actor(PartName)
					except:
						continue
					try:
						P=A.Parameter(ParmName)
						P.SetValue(ParmValue*damp)
					except:
						continue

	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

	# - - - - - - - - - - - - - - - - - -

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
